// Code generated by go generate; DO NOT EDIT.
package manifests

import (
	"net/url"

	"github.com/containers/podman/v4/pkg/bindings/internal/util"
)

// Changed returns true if named field has been set
func (o *ModifyOptions) Changed(fieldName string) bool {
	return util.Changed(o, fieldName)
}

// ToParams formats struct fields to be passed to API service
func (o *ModifyOptions) ToParams() (url.Values, error) {
	return util.ToParams(o)
}

// WithOperation set field Operation to given value
func (o *ModifyOptions) WithOperation(value string) *ModifyOptions {
	o.Operation = &value
	return o
}

// GetOperation returns value of field Operation
func (o *ModifyOptions) GetOperation() string {
	if o.Operation == nil {
		var z string
		return z
	}
	return *o.Operation
}

// WithAll set all when true, operate on all images in a manifest list that may be included in Images
func (o *ModifyOptions) WithAll(value bool) *ModifyOptions {
	o.All = &value
	return o
}

// GetAll returns value of all when true, operate on all images in a manifest list that may be included in Images
func (o *ModifyOptions) GetAll() bool {
	if o.All == nil {
		var z bool
		return z
	}
	return *o.All
}

// WithAnnotations set annotations to add to manifest list
func (o *ModifyOptions) WithAnnotations(value map[string]string) *ModifyOptions {
	o.Annotations = value
	return o
}

// GetAnnotations returns value of annotations to add to manifest list
func (o *ModifyOptions) GetAnnotations() map[string]string {
	if o.Annotations == nil {
		var z map[string]string
		return z
	}
	return o.Annotations
}

// WithArch set arch overrides the architecture for the image
func (o *ModifyOptions) WithArch(value string) *ModifyOptions {
	o.Arch = &value
	return o
}

// GetArch returns value of arch overrides the architecture for the image
func (o *ModifyOptions) GetArch() string {
	if o.Arch == nil {
		var z string
		return z
	}
	return *o.Arch
}

// WithFeatures set feature list for the image
func (o *ModifyOptions) WithFeatures(value []string) *ModifyOptions {
	o.Features = value
	return o
}

// GetFeatures returns value of feature list for the image
func (o *ModifyOptions) GetFeatures() []string {
	if o.Features == nil {
		var z []string
		return z
	}
	return o.Features
}

// WithImages set images is an optional list of images to add/remove to/from manifest list depending on operation
func (o *ModifyOptions) WithImages(value []string) *ModifyOptions {
	o.Images = value
	return o
}

// GetImages returns value of images is an optional list of images to add/remove to/from manifest list depending on operation
func (o *ModifyOptions) GetImages() []string {
	if o.Images == nil {
		var z []string
		return z
	}
	return o.Images
}

// WithOS set oS overrides the operating system for the image
func (o *ModifyOptions) WithOS(value string) *ModifyOptions {
	o.OS = &value
	return o
}

// GetOS returns value of oS overrides the operating system for the image
func (o *ModifyOptions) GetOS() string {
	if o.OS == nil {
		var z string
		return z
	}
	return *o.OS
}

// WithOSFeatures set oS features for the image
func (o *ModifyOptions) WithOSFeatures(value []string) *ModifyOptions {
	o.OSFeatures = value
	return o
}

// GetOSFeatures returns value of oS features for the image
func (o *ModifyOptions) GetOSFeatures() []string {
	if o.OSFeatures == nil {
		var z []string
		return z
	}
	return o.OSFeatures
}

// WithOSVersion set oSVersion overrides the operating system for the image
func (o *ModifyOptions) WithOSVersion(value string) *ModifyOptions {
	o.OSVersion = &value
	return o
}

// GetOSVersion returns value of oSVersion overrides the operating system for the image
func (o *ModifyOptions) GetOSVersion() string {
	if o.OSVersion == nil {
		var z string
		return z
	}
	return *o.OSVersion
}

// WithVariant set variant overrides the operating system variant for the image
func (o *ModifyOptions) WithVariant(value string) *ModifyOptions {
	o.Variant = &value
	return o
}

// GetVariant returns value of variant overrides the operating system variant for the image
func (o *ModifyOptions) GetVariant() string {
	if o.Variant == nil {
		var z string
		return z
	}
	return *o.Variant
}
