/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type InterruptModerationName string

// The valid interrupt moderation modes for I/O virtualization (IOV) offloading.
const (
	DefaultName  InterruptModerationName = "Default"
	AdaptiveName InterruptModerationName = "Adaptive"
	OffName      InterruptModerationName = "Off"
	LowName      InterruptModerationName = "Low"
	MediumName   InterruptModerationName = "Medium"
	HighName     InterruptModerationName = "High"
)

type InterruptModerationValue uint32

const (
	DefaultValue InterruptModerationValue = iota
	AdaptiveValue
	OffValue
	LowValue    InterruptModerationValue = 100
	MediumValue InterruptModerationValue = 200
	HighValue   InterruptModerationValue = 300
)

var InterruptModerationValueToName = map[InterruptModerationValue]InterruptModerationName{
	DefaultValue:  DefaultName,
	AdaptiveValue: AdaptiveName,
	OffValue:      OffName,
	LowValue:      LowName,
	MediumValue:   MediumName,
	HighValue:     HighName,
}
