// Code generated by go generate; DO NOT EDIT.
package containers

import (
	"net/url"

	"github.com/containers/podman/v4/pkg/bindings/internal/util"
)

// Changed returns true if named field has been set
func (o *CopyOptions) Changed(fieldName string) bool {
	return util.Changed(o, fieldName)
}

// ToParams formats struct fields to be passed to API service
func (o *CopyOptions) ToParams() (url.Values, error) {
	return util.ToParams(o)
}

// WithChown set field Chown to given value
func (o *CopyOptions) WithChown(value bool) *CopyOptions {
	o.Chown = &value
	return o
}

// GetChown returns value of field Chown
func (o *CopyOptions) GetChown() bool {
	if o.Chown == nil {
		var z bool
		return z
	}
	return *o.Chown
}

// WithRename set field Rename to given value
func (o *CopyOptions) WithRename(value map[string]string) *CopyOptions {
	o.Rename = value
	return o
}

// GetRename returns value of field Rename
func (o *CopyOptions) GetRename() map[string]string {
	if o.Rename == nil {
		var z map[string]string
		return z
	}
	return o.Rename
}
