// +build !linux !seccomp

package chroot

import (
	"github.com/opencontainers/runtime-spec/specs-go"
	"github.com/pkg/errors"
)

func setSeccomp(spec *specs.Spec) error {
	if spec.Linux.Seccomp != nil {
		return errors.New("configured a seccomp filter without seccomp support?")
	}
	return nil
}

func setupSeccomp(spec *specs.Spec, seccompProfilePath string) error {
	if spec.Linux != nil {
		// runtime-tools may have supplied us with a default filter
		spec.Linux.Seccomp = nil
	}
	return nil
}
