// Copyright 2019, The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// +build go1.10

package flags

// AtLeastGo110 reports whether the Go toolchain is at least Go 1.10.
const AtLeastGo110 = true
