/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type ModificationRequest struct {
	PropertyType PropertyType `json:"PropertyType,omitempty"`
	Settings     interface{}  `json:"Settings,omitempty"`
}
