/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type DeviceNamespace struct {
	RequiresDriverstore bool             `json:"requires_driverstore,omitempty"`
	DeviceCategory      []DeviceCategory `json:"device_category,omitempty"`
	DeviceInstance      []DeviceInstance `json:"device_instance,omitempty"`
}
